/**
 * NORRIS-IoC.js
 * Copyright (c) 2011 Ruben L Z Tan
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a 
 * copy of this software and associated documentation files (the "Software"), 
 * to deal in the Software without restriction, including without limitation 
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, 
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
 * IN THE SOFTWARE.
 */
'use strict';

///////////////////////////////////////////////////////////////////////////////
// @nordoc[package] ioc.installer
///////////////////////////////////////////////////////////////////////////////
module.exports.make = function make(deps) {
    var npm, fs, installer;

    npm = deps.npm,
    fs  = deps.fs;

    ///////////////////////////////////////////////////////////////////////////
    // @nordoc[module] installer
    ///////////////////////////////////////////////////////////////////////////
    installer = {

        //---------------------------------------------------------------------
        // @nordoc[property]
        //
        // @desc    Stores the basepath - the current working directory to run
        //          `npm install` from
        //---------------------------------------------------------------------
        basepath : '',

        //---------------------------------------------------------------------
        // @nordoc[method]
        //
        // @desc    Installs the NPM module
        //
        // @args    module [string]
        //          The name of the module to install
        //
        //          cb [function]
        //          The callback to call upon completion of the installation
        //          process
        //
        // @return  Returns itself
        //---------------------------------------------------------------------
        install : function install(module, cb) {
            var basepath = this.basepath;

            npm.commands.install([module], function onInstall(err, data) {
                var modpath, meta, npmMod;

                if (err) {
                    cb(err);
                    return;
                }

                try {
                    modpath = basepath + '/node_modules/' + module;
                    meta    = JSON.parse(fs.readFileSync(modpath + '/package.json', 'utf8'));

                    if (undefined === meta.main) {
                        cb(new Error('No "main" file defined. I don\'t know how to load this'));
                        return;
                    }
                    
                    npmMod = require(modpath + '/' + meta.main);
                    cb(null, npmMod);
                } catch (err) {
                    cb(err);
                }
            });

            return this;
        }
    }

    return installer;
}